/**
 * JASMINe
 * Copyright (C) 2005-2007 Bull S.A.S.
 * Contact: jasmine@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: CommandDispatcher.java 53 2007-12-14 13:41:27Z tokmensa $
 * --------------------------------------------------------------------------
 */
package org.ow2.jasmine.monitoring.mbeancmd;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.TreeMap;

/**
 * This is the main class of the archive.
 *
 * It dispatches the processing of the command to the right class.
 *
 * The available commands in the current archive are retrieved
 * by introspection of the archive.
 */
public class CommandDispatcher {
    /**
     * Dummy constructor to avoid the class from being used
     * in any other way than calling the CommandDispatcher.main method.
     */
    protected CommandDispatcher() {
        // Nothing
    }

    /**
     * Main method.
     *
     * The class is selected from the first argument.
     * If no class is available to process the command, it dispatches it
     * to the built-in help class.
     *
     * @param args arguments of the command, including the command itself at the first position.
     */
    public static void main(final String[] args) {
        String cmdName = null;
        Command cmd = null;
        String[] cmdArgs = null;

        // Obtain the list of available commands
        if (commands == null) {
            findCommands();
        }

        // Retrieve and check the command
        if (args.length > 0) {
            cmdName = args[0];
            cmd = getCommand(cmdName);
        }

        // If the command is null display help, else set arguments
        if (cmd == null) {
            cmdName = "help";
            cmd = getCommand(cmdName);
        } else if (args.length > 1) {
            cmdArgs = new String[args.length - 1];
            System.arraycopy(args, 1, cmdArgs, 0, args.length -1);
        }
        cmd.setArgs(cmdName, cmdArgs);

        // Execute command. This call will never return.
        cmd.exec();
    }

    /**
     * Returns an instance of Command.
     *
     * @param name  the name of the command.
     *
     * @return  Command instance corresponding to that name.
     */
    public static Command getCommand(final String name) {
        Command cmd = null;
        String cname = (String) commands.get(name);
        if (cname != null) {
            try {
                Class cl = Class.forName(cname);
                cmd = (Command) cl.newInstance();
            } catch (ClassNotFoundException e) {
                // Nothing to do
            } catch (InstantiationException e) {
                // Nothing to do
            } catch (IllegalAccessException e) {
                // Nothing to do
            }
        }
        return cmd;
    }

    /**
     * Returns the names of the available commands in the archive.
     *
     * @return Names of the available commands in the archive,
     *         null if no command has been set yet.
     */
    public static String[] getAvailableCommands() {
        if (commands == null) {
            return null;
        }
        String[] cmds = new String[commands.size()];
        cmds = (String[]) commands.keySet().toArray(cmds);
        return cmds;
    }

    /**
     * Gets the radix of a given function.
     *
     * @param fqn Function name.
     *
     * @return Radix of that function.
     */
    private static String getRadix(final String fqn) {
        String radix = fqn.substring(fqn.lastIndexOf(".") + 1);
        return radix;
    }

    /**
     * Retrieves the list of available commands.
     *
     * @see CommandDispatcher#commands
     */
    private static void findCommands() {
        InputStreamReader in0 = new InputStreamReader(Thread.currentThread()
                .getContextClassLoader().getResourceAsStream(COMMAND_DEFINITIONS));
        if (in0 != null) {
            BufferedReader in = new BufferedReader(in0);
            String command = null;
            try {
                while ((command = in.readLine()) != null) {
                    if ("".equals(command) || command.indexOf("#") == 0) {
                        continue;
                    }
                    Class cl;
                    try {
                        cl = Class.forName(command);
                        if (Command.class.isAssignableFrom(cl)) {
                            if (commands == null) {
                                commands = new TreeMap();
                            }
                            commands.put(getRadix(command).toLowerCase(), command);
                        }
                    } catch (ClassNotFoundException e) {
                        e.printStackTrace(System.err);
                    }
                }
            } catch (IOException e) {
                System.err.println("Cannot access " + COMMAND_DEFINITIONS);
                e.printStackTrace();
            }
        }

    }

    /**
     * File containing all command definitions.
     */
    private static final String COMMAND_DEFINITIONS = "META-INF/mbeancmd/commands.defs";

    /**
     * List of commands.
     */
    private static TreeMap commands = null;
}
