/**
 * JASMINe
 * Copyright (C) 2005-2007 Bull S.A.S.
 * Contact: jasmine@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: JmxAP.java 41 2007-12-13 14:17:00Z tokmensa $
 * --------------------------------------------------------------------------
 */
package org.ow2.jasmine.monitoring.mbeancmd;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.util.Enumeration;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Properties;
import java.util.TreeMap;

import javax.management.MBeanServerConnection;

/**
 * A JMX Access Point.
 */
public class JmxAP {
    /**
     * Creates a JmxAP that connects to a URL.
     *
     * @param jmxUrl  URL to connect to.
     */
    public JmxAP(final String jmxUrl) {
        this.jmxUrl = jmxUrl;
    }

    /**
     * Creates a JmxAP that connects to the default URL.
     */
    public JmxAP() {
        this((String) jmxTargets.get(defaultTarget));
    }

    /**
     * @return  The String version of the JMX URL connected to.
     */
    public String getJmxUrl() {
        return jmxUrl;
    }

    /**
     * @return  The MBeanServerConnection instance used to connect to the J2EE
     *          server via JMX.
     */
    public MBeanServerConnection getMBeanServerConnection() {
        if(jmxConnection == null) {
            jmxConnection = new JmxCnxWrapper(jmxUrl, (String) jmxUsers.get(jmxUrl), (String) jmxPasswords.get(jmxUrl));
        }
        return jmxConnection;
    }

    /**
     * @param propertyFile  File name of the properties file.
     *
     * @return  Parsed version of the propertyFile file.
     *
     * @throws IOException  If propertyFile cannot be read.
     */
    private static Properties getProperties(final File propertyFile) throws IOException {
        Properties properties = new Properties();

        FileInputStream in = new FileInputStream(propertyFile);
        properties.load(in);
        in.close();

        return properties;
    }

    /**
     * Gets the JMX URL for a given server name.
     *
     * @param name  Name of the server.
     *
     * @return  JMX URL corresponding to that server name.
     */
    public static String getJmxUrl(final String name) {
        return (String) jmxTargets.get(name);
    }

    /**
     * @return  An array containing the names of all current JMX targets.
     */
    public static String[] getJmxTargets() {

        String[] targets = null;
        if (!jmxTargets.isEmpty()) {
            targets = new String[jmxTargets.size()];
            targets = (String[]) jmxTargets.keySet().toArray(targets);
        }
        return targets;
    }

    /**
     * Gets the names of all JMX targets matching a certain criteria.
     *
     * @param proposed  Criteria.
     *
     * @return  An array containing the names of all current JMX targets.
     */
    public static String[] getJmxTargets(final String[] proposed) {
        boolean all = false;
        String[] targetsArray = null;
        List targetsList = new LinkedList();

        // Look for defined target
        if (proposed != null) {
            for (int i = 0; i < proposed.length; i++) {
                if ("all".equalsIgnoreCase(proposed[i])) {
                    all = true;
                    break;
                }
                String url = getJmxUrl(proposed[i]);
                if (url != null) {
                    targetsList.add(proposed[i]);
                }
            }
        } else {
            if (defaultTarget != null) {
                targetsList.add(defaultTarget);
            }
        }

        if (all) {
            targetsArray = getJmxTargets();
        } else {
            if (targetsList.size() > 0) {
                targetsArray = new String[targetsList.size()];
                for (int i = 0; i < targetsList.size(); i++) {
                     targetsArray[i] = (String) targetsList.get(i);
                }
            }
        }
        return targetsArray;
    }

    /**
     * @return  Default target.
     */
    public static String getDefaultTarget() {
        return defaultTarget;
    }

    /**
     * Loads all JMX jmxConnection properties from a file.
     */
    private static void load() {
        // Default target is derived from the jasmine.jmx.url. It may also be
        // defined in the property file using the jasmine.jmx.default.target
        // property. If both are set, jasmine.jmx.url takes precedence.
        String dftTarget = null;
        String url = System.getProperty("jasmine.jmx.url");
        if (url == null) {
            url = System.getProperty("jonas.jmx.url");
            if (url != null) {
                System.err.println("WARNING: property jonas.jmx.url is deprecated. Use jasmine.jmx.url instead.");
            }
        }
        String user = System.getProperty("jasmine.jmx.user");
        if (user == null) {
            user = System.getProperty("jonas.jmx.user");
            if (user != null) {
                System.err.println("WARNING: property jonas.jmx.user is deprecated. Use jasmine.jmx.user instead.");
            }
        }
        String password = System.getProperty("jasmine.jmx.password");
        if (password == null) {
            password = System.getProperty("jonas.jmx.password");
            if (password != null) {
                System.err.println("WARNING: property jonas.jmx.password is deprecated. Use jasmine.jmx.password instead.");
            }
        }
        String filename = System.getProperty("jasmine.jmx.file");
        if (filename == null) {
            filename = System.getProperty("jonas.jmx.file");
            if (filename != null) {
                System.err.println("WARNING: property jonas.jmx.file is deprecated. Use jasmine.jmx.file instead.");
            } else {
                filename = DEFAULT_FILE;
            }
        }
        String name = null;
        if (url != null) {
            name = "jonas";
            if (url.matches(".*connector_.*")) {
                int ix = url.lastIndexOf("connector_");
                name = url.substring(ix + "connector_".length());
            }
            jmxTargets.put(name, url);
            if(user != null && password != null) {
                jmxUsers.put(url, user);
                jmxPasswords.put(user, password);
            }
            defaultTarget = name;
        }

        // Load property file
        Properties props = null;
        try {
            props = getProperties(new File(filename));
        } catch (IOException e) {
            System.err.println("WARNING : cannot read default JMX configuration file : " + e);
        }

        if ((props != null) && (props.size() != 0)) {
            // Parse the property file
            Enumeration en = props.propertyNames();
            while (en.hasMoreElements()) {
                String key = (String) en.nextElement();
                if (key.startsWith("jasmine.jmx.url.")) {
                    name = key.substring("jasmine.jmx.url.".length());
                    url = props.getProperty(key);
                    jmxTargets.put(name, url);
                } else if (key.startsWith("jasmine.jmx.user.")) {
                    name = key.substring("jasmine.jmx.user.".length());
                    url = (String) jmxTargets.get(name);
                    if(url == null) {
                        System.err.println("WARNING : cannot set user for server : " + name);
                        continue;
                    }
                    jmxUsers.put(url, key);
                } else if (key.startsWith("jasmine.jmx.password.")) {
                    name = key.substring("jasmine.jmx.password.".length());
                    url = (String) jmxTargets.get(name);
                    if(url == null) {
                        System.err.println("WARNING : cannot set password for server : " + name);
                        continue;
                    }
                    jmxPasswords.put(url, key);
                } else if ("jasmine.jmx.default.target".equals(key)) {
                    if (defaultTarget == null) {
                        dftTarget = props.getProperty(key);
                    }
                } else if (key.startsWith("jonas.jmx.url.")) {
                    name = key.substring("jonas.jmx.url.".length());
                    url = props.getProperty(key);
                    jmxTargets.put(name, url);
                    System.err.println("WARNING: key jonas.jmx.url." + name + " in configuration file " + filename
                            + " is deprecated. Use jasmine.jmx.url." + name + " instead.");
                } else if (key.startsWith("jonas.jmx.user.")) {
                    name = key.substring("jonas.jmx.user.".length());
                    url = (String) jmxTargets.get(name);
                    if(url == null) {
                        System.err.println("WARNING : cannot set user for server : " + name);
                        continue;
                    }
                    jmxUsers.put(url, key);
                    System.err.println("WARNING: key jonas.jmx.user." + name + " in configuration file " + filename
                            + " is deprecated. Use jasmine.jmx.user." + name + " instead.");
                } else if (key.startsWith("jonas.jmx.password.")) {
                    name = key.substring("jonas.jmx.password.".length());
                    url = (String) jmxTargets.get(name);
                    if(url == null) {
                        System.err.println("WARNING : cannot set password for server : " + name);
                        continue;
                    }
                    jmxPasswords.put(url, key);
                    System.err.println("WARNING: key jonas.jmx.password." + name + " in configuration file " + filename
                            + " is deprecated. Use jasmine.jmx.password." + name + " instead.");
                } else if ("jonas.jmx.default.target".equals(key)) {
                    if (defaultTarget == null) {
                        dftTarget = props.getProperty(key);
                    }
                    System.err.println("WARNING: key jonas.jmx.default.target in configuration file " + filename
                            + " is deprecated. Use jasmine.jmx.default.target instead.");
                }
            }
        }

        // Check that the default target belongs to the targets
        if (defaultTarget == null) {
            if (dftTarget != null) {
                if (jmxTargets.get(dftTarget) != null) {
                    defaultTarget = dftTarget;
                } else {
                    System.err
                            .println("WARNING : jasmine.jmx.default.target specifies an invalid default target "
                                    + dftTarget + ". No default target.");
                }
            } else {
                // use default URL as default target
                defaultTarget = "jonas";
                jmxTargets.put(defaultTarget, DEFAULT_URL);
                System.err
                        .println("WARNING : jasmine.jmx.default.target not specified => adding the default jmxUrl : "
                                + defaultTarget + " jmxUrl=" + DEFAULT_URL);
            }
        }
    }

    /**
     * Current JMX jmxConnection.
     */
    private MBeanServerConnection jmxConnection = null;

    /**
     * Current JMX URL.
     */
    private String jmxUrl = DEFAULT_URL;

    /**
     * Map of JMX targets.
     */
    private static Map jmxTargets = new TreeMap();

    /**
     * Map of JMX user names.
     */
    private static Map jmxUsers = new TreeMap();

    /**
     * Map of JMX passwords.
     */
    private static Map jmxPasswords = new TreeMap();

    /**
     * Default JMX target.
     */
    private static String defaultTarget = null;

    /**
     * Default JMX URL.
     */
    public static final String DEFAULT_URL = "service:jmx:rmi:///jndi/rmi://localhost:1099/jrmpconnector_jonas";

    /**
     * Default name of the properties file.
     */
    public static final String DEFAULT_FILE = "jmxurls.properties";

    static {
        load();
    }
}
