/**
 * JASMINe
 * Copyright (C) 2005-2007 Bull S.A.S.
 * Contact: jasmine@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: XServerSampler.java 55 2007-12-17 16:03:02Z tokmensa $
 * --------------------------------------------------------------------------
 */
package org.ow2.jasmine.monitoring.mbeancmd.sampling;

import java.io.PrintStream;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.TreeMap;
import java.util.Map.Entry;

/**
 * Server sampler.
 */
public class XServerSampler {
    /**
     * Constructor: saves the sampler type.
     *
     * @param type  Sampler type
     */
    public XServerSampler(final String type) {
        sampleType = type;
    }

    /**
     * Adds a manangee.
     *
     * @param name    Managee name.
     * @param jmxUrl  JMX URL for managee.
     */
    public void addManagee(final String name, final String jmxUrl) {
        manageeURLs.put(name, jmxUrl);
    }

    /**
     * @param objectNameAsString  Object name filter pattern.
     */
    public void setOnPattern(final String objectNameAsString) {
        onPattern = objectNameAsString;
    }

    /**
     * Start processing. Will never return, except on failure.
     *
     * @param period  Poll period.
     * @param out     Stream to output results into.
     */
    public void process(final long period, final PrintStream out) {
        List ss = new LinkedList();
        Iterator it = manageeURLs.entrySet().iterator();
        while (it.hasNext()) {
            Entry e = (Entry) it.next();
            SamplerContext ctx = new SamplerContext();
            ctx.name = (String) e.getKey();
            ctx.sampler = SamplerFactory.newSampler(sampleType, (String) e.getValue());
            if (onPattern != null) {
                ctx.sampler.setOn(onPattern);
            }
            ss.add(ctx);
        }
        context = (SamplerContext[]) ss.toArray(new SamplerContext[ss.size()]);

        // Initialize samplers
        for (int i = 0; i < context.length; i++) {
            context[i].sampler.init();
        }

        // Infinite sampling loop
        boolean printHeader = true;
        while (true) {
            SamplerContext ctx = null;
            SampleData data = null;
            // TODO: create one thread per polled server
            for (int i = 0; i < context.length; i++) {
                ctx = context[i];
                data = null;
                try {
                    data = ctx.sampler.sample(ctx.previousData);
                    if (data != null) {
                        if (printHeader) {
                            out.println(data.getPrintHeader());
                            printHeader = false;
                        }
                        data.printData(out);
                    }

                    if (!ctx.started) {
                        // first line contains data header
                        ctx.started = true;
                    }
                } catch (Exception e) {
                    e.printStackTrace(System.err);
                    if (ctx.started) {
                        System.err.println("Error sampling managee " + ctx.name);
                    }
                }
                ctx.previousData = data;
            }
            try {
                Thread.sleep(period * 1000);
            } catch (Exception e) {
                // Nothing
            }
        }

    }

    /**
     * Sampler context.
     */
    private class SamplerContext {
        /**
         * Managee name.
         */
        private String name = null;

        /**
         * Sampler.
         */
        private Sampler sampler = null;

        /**
         * Previous data.
         */
        private SampleData previousData = null;

        /**
         * Sampler started ?
         */
        private boolean started = false;
    }

    /**
     * URLs of managees.
     */
    private Map manageeURLs = new TreeMap();

    /**
     * Sample type.
     */
    private String sampleType = null;

    /**
     * Object name pattern.
     */
    private String onPattern = null;

    /**
     * Array of contexts.
     */
    private SamplerContext[] context = null;
}
